@@ -1,43 +1,42 @@
-"""
-Incremental regression
-"""
+"""The :mod:`incremental_regression` contains IncrementalRegression."""
 
-# Authors: Peter Steiner <peter.steiner@tu-dresden.de>, Azarakhsh Jalalvand <azarakhsh.jalalvand@ugent.be>
+# Authors: Peter Steiner <peter.steiner@tu-dresden.de>
 # License: BSD 3 clause
-
 import sys
+from typing import Union, cast
 
 import numpy as np
-import scipy
 from sklearn.base import BaseEstimator, RegressorMixin
+from sklearn.utils.validation import _deprecate_positional_args
 from sklearn.utils.extmath import safe_sparse_dot
 from sklearn.preprocessing import StandardScaler
 from sklearn.exceptions import NotFittedError
-from joblib import Parallel, delayed
 
 
 class IncrementalRegression(BaseEstimator, RegressorMixin):
     """
     Linear regression.
+
     This linear regression algorithm is able to perform a linear regression
     with the L2 regularization and iterative fit. [1]
     .. [1] https://ieeexplore.ieee.org/document/4012031
-    
+
     References
     ----------
     N. Liang, G. Huang, P. Saratchandran and N. Sundararajan,
     "A Fast and Accurate Online Sequential Learning Algorithm for Feedforward Networks,"
-    in IEEE Transactions on Neural Networks, vol. 17, no. 6, pp. 1411-1423, Nov. 2006, doi: 10.1109/TNN.2006.880583.
-    
+    in IEEE Transactions on Neural Networks, vol. 17, no. 6, pp. 1411-1423,
+    Nov. 2006, doi: 10.1109/TNN.2006.880583.
+
     Parameters
     ----------
-    alpha : float, default=1.0
+    alpha : float, default=1e-5
         L2 regularization parameter
     fit_intercept : bool, default=True
         Fits a constant offset if True. Use this if input values are not average free.
     normalize : bool, default=False
         Performs a preprocessing normalization if True.
-    
+
     Attributes
     ----------
     coef_ : array, shape (n_features,) or (n_targets, n_features)
@@ -46,31 +45,46 @@ class IncrementalRegression(BaseEstimator, RegressorMixin):
         Independent term in decision function. Set to 0.0 if
         ``fit_intercept = False``.
     """
-    def __init__(self, alpha=1e-5, fit_intercept=True, normalize=False):
+
+    @_deprecate_positional_args
+    def __init__(self, *, alpha: float = 1e-5,
+                 fit_intercept: bool = True,
+                 normalize: bool = False):
+        """Construct the IncrementalRegression."""
         self.alpha = alpha
         self.fit_intercept = fit_intercept
         self.normalize = normalize
         self.scaler = StandardScaler(copy=False)
 
-        self._K = None
-        self._xTy = None
-        self._output_weights = None
+        self._K: np.ndarray = np.ndarray([])
+        self._xTy: np.ndarray = np.ndarray([])
+        self._output_weights: np.ndarray = np.ndarray([])
+
+    def partial_fit(self, X: np.ndarray, y: np.ndarray,
+                    partial_normalize: bool = True,
+                    reset: bool = False, validate: bool = True,
+                    postpone_inverse: bool = False) -> BaseEstimator:
+        """
+        Fit the regressor partially.
 
-    def partial_fit(self, X, y, partial_normalize=True, reset=False, validate=True, postpone_inverse=False, n_jobs=None):
-        """Fits the regressor partially.
         Parameters
         ----------
-        X : {ndarray, sparse matrix} of shape (n_samples, n_features)
-        y : {ndarray, sparse matrix} of shape (n_samples,) or (n_samples, n_targets)
+        X : ndarray of shape (samples, n_features)
+        y : ndarray of shape (n_samples,) or (n_samples, n_targets)
+            The targets to predict.
         partial_normalize : bool, default=True
             Partial fits the normalization transformer on this sample if True.
         reset : bool, default=False
             Begin a new fit, drop prior fits.
         validate: bool, default=True
             Validate input data if True.
+        postpone_inverse : bool, default=False
+            If the output weights have not been fitted yet, regressor might be hinted at
+            postponing inverse calculation.
+
         Returns
         -------
-        self
+        self : returns a partially fitted IncrementalRegression model
         """
         if validate:
             self._validate_data(X, y, multi_output=True)
@@ -78,76 +92,95 @@ class IncrementalRegression(BaseEstimator, RegressorMixin):
         X_preprocessed = self._preprocessing(X, partial_normalize=partial_normalize)
 
         if reset:
-            self._K = None
-            self._xTy = None
-            self._output_weights = None
+            self._K = np.ndarray([])
+            self._xTy = np.ndarray([])
+            self._output_weights = np.ndarray([])
 
-        if self._K is None:
+        if self._K.shape == ():
             self._K = safe_sparse_dot(X_preprocessed.T, X_preprocessed)
         else:
             self._K += safe_sparse_dot(X_preprocessed.T, X_preprocessed)
 
-        if self._xTy is None:
+        if self._xTy.shape == ():
             self._xTy = safe_sparse_dot(X_preprocessed.T, y)
         else:
             self._xTy += safe_sparse_dot(X_preprocessed.T, y)
 
         # can only be postponed if output weights have not been initialized yet
-        if postpone_inverse and self._output_weights is None:
+        if postpone_inverse and self._output_weights.shape == ():
             return self
 
         P = np.linalg.inv(self._K + self.alpha * np.identity(X_preprocessed.shape[1]))
 
-        if self._output_weights is None:
-            if n_jobs is None:
-                self._output_weights = np.matmul(P, self._xTy)
-            else:
-                self._output_weights = np.asarray(Parallel(n_jobs=n_jobs)(delayed(safe_sparse_dot)(P, self._xTy[:, k]) for k in range(self._xTy.shape[1]))).T
+        if self._output_weights.shape == ():
+            self._output_weights = np.matmul(P, cast(np.ndarray, self._xTy))
         else:
-            if n_jobs is None:
-                self._output_weights += np.matmul(P, safe_sparse_dot(X_preprocessed.T, (y - safe_sparse_dot(X_preprocessed, self._output_weights))))
-            else:
-                self._output_weights += np.asarray(Parallel(n_jobs=n_jobs)(delayed(safe_sparse_dot)(P, safe_sparse_dot(X_preprocessed.T, (y[:, k] - safe_sparse_dot(X_preprocessed, self._output_weights[:, k])))) for k in range(self._xTy.shape[1]))).T
-            # self._output_weights += np.matmul(P, self._xTy - np.matmul(self._K, self._output_weights))
+            self._output_weights += np.matmul(
+                P, safe_sparse_dot(X_preprocessed.T,
+                                   (y - safe_sparse_dot(X_preprocessed,
+                                                        self._output_weights))))
+            # self._output_weights += np.matmul(
+            #     P, self._xTy - np.matmul(self._K, self._output_weights))
         return self
 
-    def fit(self, X, y):
-        """Fits the regressor.
+    def fit(self, X: np.ndarray, y: np.ndarray) -> BaseEstimator:
+        """
+        Fit the regressor.
+
         Parameters
         ----------
-        X : {ndarray, sparse matrix} of shape (n_samples, n_features)
-        y : {ndarray, sparse matrix} of shape (n_samples,) or (n_samples, n_targets)
+        X : ndarray of shape (samples, n_features)
+        y : ndarray of shape (n_samples,) or (n_samples, n_targets)
+            The targets to predict.
+        partial_normalize : bool, default=True
+            Partial fits the normalization transformer on this sample if True.
+        reset : bool, default=False
+            Begin a new fit, drop prior fits.
+        validate: bool, default=True
+            Validate input data if True.
+        postpone_inverse : bool, default=False
+            If the output weights have not been fitted yet, regressor might be hinted at
+            postponing inverse calculation.
+
         Returns
         -------
-        self
+        self : returns a fitted IncrementalRegression model
         """
         self.partial_fit(X, y, partial_normalize=False, reset=True, validate=True)
         return self
 
-    def predict(self, X):
-        """Predicts output y according to input X.
+    def predict(self, X: np.ndarray) -> np.ndarray:
+        """Predict output y according to input X.
+
         Parameters
         ----------
-        X : {ndarray, sparse matrix} of shape (n_samples, n_features)
+        X : ndarray of shape (samples, n_features)
+
         Returns
         -------
-        Y : ndarray of shape (n_samples,) or (n_samples, n_targets)
+        y : ndarray of shape (n_samples,) or (n_samples, n_targets)
         """
-        if self._output_weights is None:
+        if self._output_weights.shape == ():
             raise NotFittedError(self)
 
-        return safe_sparse_dot(self._preprocessing(X, partial_normalize=False), self._output_weights)
+        return safe_sparse_dot(self._preprocessing(X, partial_normalize=False),
+                               self._output_weights)
+
+    def _preprocessing(self, X: np.ndarray,
+                       partial_normalize: bool = True) -> np.ndarray:
+        """
+        Apply preprocessing on the input data X.
 
-    def _preprocessing(self, X, partial_normalize=True):
-        """Applies preprocessing on the input data X.
         Parameters
         ----------
-        X : {ndarray, sparse matrix} of shape (n_samples, n_features)
+        X : ndarray of shape (samples, n_features)
         partial_normalize : bool, default=True
             Partial fits the normalization transformer on this sample if True.
+
         Returns
         -------
-        X_preprocessed : {ndarray, sparse matrix} of shape (n_samples, n_features) or (n_samples, n_features+1)
+        X_preprocessed : ndarray of shape (n_samples, n_features)
+        or (n_samples, n_features+1)
         """
         X_preprocessed = X
 
@@ -162,12 +195,14 @@ class IncrementalRegression(BaseEstimator, RegressorMixin):
 
         return X_preprocessed
 
-    def __sizeof__(self):
-        """Returns the size of the object in bytes.
+    def __sizeof__(self) -> int:
+        """
+        Return the size of the object in bytes.
+
         Returns
         -------
         size : int
-        Object memory in bytes.
+            Object memory in bytes.
         """
         return object.__sizeof__(self) + \
             self._K.nbytes + \
@@ -176,14 +211,18 @@ class IncrementalRegression(BaseEstimator, RegressorMixin):
             sys.getsizeof(self.scaler)
 
     @property
-    def coef_(self):
-        """Returns the output weights without intercept. Compatibility to sklearn.linear_model.Ridge.
+    def coef_(self) -> Union[np.ndarray, None]:
+        """
+        Return the output weights without intercept.
+
+        Compatibility to ```sklearn.linear_model.Ridge```.
+
         Returns
         -------
-        coef_ : array, shape (n_features,) or (n_targets, n_features)
+        coef_ : ndarray of shape (n_features,) or (n_targets, n_features)
             Weight vector(s).
         """
-        if self._output_weights is not None:
+        if self._output_weights.shape != ():
             if self.fit_intercept:
                 if self._output_weights.ndim == 2:
                     return np.transpose(self._output_weights)[:, :-1]
@@ -191,14 +230,20 @@ class IncrementalRegression(BaseEstimator, RegressorMixin):
                     return self._output_weights[:-1]
             else:
                 return np.transpose(self._output_weights)
+        return self._output_weights
 
     @property
-    def intercept_(self):
-        """Returns the intercept of output output weights. Compatibility to sklearn.linear_model.Ridge.
+    def intercept_(self) -> Union[float, np.ndarray]:
+        """
+        Return the intercept of output output weights.
+
+        Compatibility to ```sklearn.linear_model.Ridge```.
+
         Returns
         -------
-        intercept_ : float | array, shape = (n_targets,)
+        intercept_ : Union[float, np.ndarray]
             Independent term in decision function.
         """
-        if self._output_weights is not None and self.fit_intercept:
+        if self._output_weights.shape != () and self.fit_intercept:
             return np.transpose(self._output_weights)[:, -1]
+        return np.array([])
