@@ -1,9 +1,8 @@
-"""
-The :mod:`pyrcn.coates` module implements the coates preprocessing.
-"""
+"""The :mod:`pyrcn.postprocessing` module provides postprocessing methods."""
 
 # See https://github.com/TUD-STKS/PyRCN and for documentation.
 
 from pyrcn.postprocessing._normal_distribution import NormalDistribution
 
-__all__ = ['NormalDistribution']
+
+__all__ = ('NormalDistribution',)
